/**
 * This piece of code demonstrates the thread organization in CUDA
 */

#include <cuda_runtime.h>
#include <stdio.h>
#include <stdlib.h>

#include "../misc/cuda_util.h"

#define BLOCKSIZE 4
#define NUM_BLOCKS 2


__global__ void init_data(int* global, int* block, int* thread)
{

   int id = blockIdx.x * blockDim.x + threadIdx.x;

   global[id] = id;
   block[id]  = blockIdx.x;
   thread[id] = threadIdx.x;

}


int main(int argc, char** argv)
{
   int num_elements = NUM_BLOCKS * BLOCKSIZE;
   int size = sizeof(int) * num_elements;


   //allocate host memory
   int *global_h, *block_h, *thread_h;
   global_h = (int*) malloc(size);
   block_h  = (int*) malloc(size);
   thread_h = (int*) malloc(size);


   //allocate device memory
   int *global_d, *block_d, *thread_d;
   HANDLE_ERROR( cudaMalloc(&global_d, size) );  //sync
   HANDLE_ERROR( cudaMalloc(&block_d, size) );  //sync
   HANDLE_ERROR( cudaMalloc(&thread_d, size) );  //sync


   //start kernel
   init_data<<<NUM_BLOCKS, BLOCKSIZE>>>(global_d, block_d, thread_d); //async!
   HANDLE_ERROR(cudaGetLastError());


   //copy data from device to host
   HANDLE_ERROR( cudaMemcpy(global_h, global_d, size, cudaMemcpyDeviceToHost)); //sync
   HANDLE_ERROR( cudaMemcpy(block_h,  block_d,  size, cudaMemcpyDeviceToHost)); //sync
   HANDLE_ERROR( cudaMemcpy(thread_h, thread_d, size, cudaMemcpyDeviceToHost)); //sync


   //print the result
   int i;
   printf("\nglobal:\t");
   for(i=0;i < num_elements; ++i)
      printf("%d ",global_h[i]);
   printf("\n\n");
   printf("block:\t");
   for(i=0;i < num_elements; ++i)
      printf("%d ",block_h[i]);
   printf("\n\n");
   printf("thread:\t");
   for(i=0;i < num_elements; ++i)
      printf("%d ",thread_h[i]);
   printf("\n\n");


   //free memory
   free(global_h); free(block_h); free(thread_h);
   cudaFree(global_d); cudaFree(block_d); cudaFree(thread_d);

   return 0;
}
